package com.jiuqi.grid;

import com.jiuqi.util.StreamException;
import com.jiuqi.util.Stream;
import java.util.Collection;
import java.util.TreeSet;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Set;
import com.jiuqi.util.Matrix;
import java.util.List;
import java.io.Serializable;

public final class GridFieldList implements Serializable
{
    private List list;
    private GridData parent;
    private boolean autoMerge;
    private Matrix rectMaps;
    private Set tmpRects;
    
    @Deprecated
    public GridFieldList(final GridData aParent) {
        this();
        this.parent = aParent;
    }
    
    public GridFieldList() {
        final Comparator rectCmp = new Comparator() {
            public int compare(final Object o1, final Object o2) {
                final CellField f1 = (CellField)o1;
                final CellField f2 = (CellField)o2;
                return f1.compareTo(f2);
            }
        };
        this.list = new ArrayList();
        this.rectMaps = new Matrix();
        this.tmpRects = new TreeSet(rectCmp);
    }
    
    public int count() {
        return this.list.size();
    }
    
    @Deprecated
    public GridData getParent() {
        return this.parent;
    }
    
    public CellField get(final int index) {
        return this.list.get(index);
    }
    
    public void remove(final int index) {
        final CellField field = this.list.remove(index);
        for (int col = field.left; col <= field.right; ++col) {
            for (int row = field.top; row <= field.bottom; ++row) {
                this.rectMaps.setItem(col, row, (Object)null);
            }
        }
    }
    
    protected boolean remove(final CellField field) {
        for (int i = 0; i < this.list.size(); ++i) {
            if (field == this.list.get(i)) {
                this.remove(i);
                return true;
            }
        }
        return false;
    }
    
    public CellField getMergeRect(final int col, final int row) {
        if (col >= this.rectMaps.getColCount() || row >= this.rectMaps.getRowCount()) {
            return null;
        }
        return (CellField)this.rectMaps.getItem(col, row);
    }
    
    public boolean checkCanMerge(final CellField field) {
        for (int col = field.left; col <= field.right; ++col) {
            for (int row = field.top; row <= field.bottom; ++row) {
                final CellField f = this.getMergeRect(col, row);
                if (f != null) {
                    switch (CellField.compareField(f, field)) {
                        case -2:
                        case 1: {
                            return false;
                        }
                    }
                }
            }
        }
        return true;
    }
    
    public boolean addMergeRect(final CellField field) {
        if (this.checkCanMerge(field)) {
            this.tmpRects.clear();
            if (this.rectMaps.getColCount() <= field.right) {
                this.rectMaps.setColCount(field.right + 1);
            }
            if (this.rectMaps.getRowCount() <= field.bottom) {
                this.rectMaps.setRowCount(field.bottom + 1);
            }
            for (int col = field.left; col <= field.right; ++col) {
                for (int row = field.top; row <= field.bottom; ++row) {
                    final CellField f = (CellField)this.rectMaps.getItem(col, row);
                    if (f != null && !this.tmpRects.contains(f)) {
                        this.tmpRects.add(f);
                    }
                    this.rectMaps.setItem(col, row, (Object)field);
                }
            }
            if (!this.tmpRects.isEmpty()) {
                this.list.removeAll(this.tmpRects);
                this.tmpRects.clear();
            }
            this.list.add(field);
            return true;
        }
        return false;
    }
    
    public int removeMergeRect(final CellField field) {
        int count = 0;
        for (int col = field.left; col <= field.right; ++col) {
            for (int row = field.top; row <= field.bottom; ++row) {
                final CellField f = this.getMergeRect(col, row);
                if (f != null) {
                    if (field.containField(f)) {
                        this.remove(f);
                        ++count;
                    }
                    else {
                        final GridCell originCell = this.parent.getCellEx(f.left, f.top);
                        if (f.left <= field.right && f.right > field.right) {
                            f.left = field.right + 1;
                        }
                        if (f.top <= field.bottom && f.bottom > field.bottom) {
                            f.top = field.bottom + 1;
                        }
                        final GridCell currentCell = this.parent.getCellEx(f.left, f.top);
                        if (originCell != null && currentCell != null && !originCell.equals(currentCell)) {
                            currentCell.setBackColor(originCell.getBackColor());
                            currentCell.setBackStyle(originCell.getBackStyle());
                            currentCell.setLEdgeStyle(originCell.getLEdgeStyle());
                            currentCell.setLEdgeColor(originCell.getLEdgeColor());
                            currentCell.setTEdgeStyle(originCell.getTEdgeStyle());
                            currentCell.setTEdgeColor(originCell.getTEdgeColor());
                            currentCell.setREdgeStyle(originCell.getREdgeStyle());
                            currentCell.setREdgeColor(originCell.getREdgeColor());
                            currentCell.setBEdgeStyle(originCell.getBEdgeStyle());
                            currentCell.setBEdgeColor(originCell.getBEdgeColor());
                            currentCell.setShowText("");
                            this.parent.setCell(currentCell);
                        }
                    }
                }
            }
        }
        return count;
    }
    
    public void delRows(final int startRow, final int count) {
        final int top = startRow;
        final int bottom = startRow + count - 1;
        for (int i = this.count() - 1; i >= 0; --i) {
            final CellField f = this.get(i);
            final int t = (f.top < top) ? top : f.top;
            final int b = (f.bottom < bottom) ? f.bottom : bottom;
            if (b >= t) {
                f.bottom -= b - t + 1;
                if (f.bottom < f.top) {
                    this.remove(i);
                }
            }
            else if (f.top > bottom) {
                f.top -= count;
                f.bottom -= count;
            }
        }
        if (startRow < this.rectMaps.getRowCount()) {
            this.rectMaps.deleteRow(startRow, count);
        }
    }
    
    public void delCols(final int startCol, final int count) {
        final int left = startCol;
        final int right = startCol + count - 1;
        for (int i = this.count() - 1; i >= 0; --i) {
            final CellField f = this.get(i);
            final int l = (f.left < left) ? left : f.left;
            final int r = (f.right < right) ? f.right : right;
            if (r >= l) {
                f.right -= r - l + 1;
            }
            if (f.right < f.left) {
                this.remove(i);
            }
            else if (f.left > right) {
                f.left -= count;
                f.right -= count;
            }
        }
        if (startCol < this.rectMaps.getColCount()) {
            this.rectMaps.deleteCol(startCol, count);
        }
    }
    
    public void insertRows(final int startRow, final int count) {
        Object[] colItems = null;
        final List adjRects = new ArrayList();
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (startRow > f.top && startRow <= f.bottom) {
                f.bottom += count;
                if (colItems == null) {
                    colItems = new Object[this.rectMaps.getColCount()];
                }
                for (int col = f.left; col <= f.right; ++col) {
                    colItems[col] = f;
                }
            }
            else if (startRow <= f.top) {
                f.top += count;
                f.bottom += count;
            }
            else if (startRow == f.bottom + 1 && f.right > f.left && this.autoMerge) {
                adjRects.add(f);
            }
        }
        if (this.rectMaps.getRowCount() > 0 && startRow <= this.rectMaps.getRowCount()) {
            this.rectMaps.insertRow(startRow, count, colItems);
        }
        for (int i = 0; i < adjRects.size(); ++i) {
            final CellField f = adjRects.get(i);
            for (int row = startRow; row < startRow + count; ++row) {
                this.addMergeRect(new CellField(f.left, row, f.right, row));
            }
        }
    }
    
    public void insertCols(final int startCol, final int count) {
        Object[] rowItems = null;
        final List adjRects = new ArrayList();
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (startCol > f.left && startCol <= f.right) {
                f.right += count;
                if (rowItems == null) {
                    rowItems = new Object[this.rectMaps.getRowCount()];
                }
                for (int row = f.top; row <= f.bottom; ++row) {
                    rowItems[row] = f;
                }
            }
            else if (startCol <= f.left) {
                f.left += count;
                f.right += count;
            }
            else if (startCol == f.right + 1 && f.top < f.bottom && this.autoMerge) {
                adjRects.add(f);
            }
        }
        if (this.rectMaps.getColCount() > 0 && startCol <= this.rectMaps.getColCount()) {
            this.rectMaps.insertCol(startCol, count, rowItems);
        }
        for (int i = 0; i < adjRects.size(); ++i) {
            final CellField f = adjRects.get(i);
            for (int col = startCol; col < startCol + count; ++col) {
                this.addMergeRect(new CellField(col, f.top, col, f.bottom));
            }
        }
    }
    
    public void extendRows(final int startRow, final int count) {
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (startRow >= f.top && startRow <= f.bottom) {
                f.bottom += count;
            }
            else if (startRow < f.top) {
                f.top += count;
                f.bottom += count;
            }
        }
        if (this.rectMaps.getRowCount() > 0 && startRow < this.rectMaps.getRowCount()) {
            this.rectMaps.duplicateRow(startRow, count);
        }
    }
    
    public void extendCols(final int startCol, final int count) {
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (startCol >= f.left && startCol <= f.right) {
                f.right += count;
            }
            else if (startCol < f.left) {
                f.left += count;
                f.right += count;
            }
        }
        if (this.rectMaps.getColCount() > 0 && startCol < this.rectMaps.getColCount()) {
            this.rectMaps.duplicateCol(startCol, count);
        }
    }
    
    public List findFieldsInCols(final int from, final int to) {
        final List result = new ArrayList();
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (f.left >= from && f.right <= to) {
                result.add(f);
            }
        }
        return result;
    }
    
    public List findFieldsInRows(final int from, final int to) {
        final List result = new ArrayList();
        for (int i = 0; i < this.count(); ++i) {
            final CellField f = this.get(i);
            if (f.top >= from && f.bottom <= to) {
                result.add(f);
            }
        }
        return result;
    }
    
    public void loadFromStream(final Stream stream, final int size) throws StreamException {
        this.list.clear();
        this.rectMaps.clear();
        for (int i = 0; i < size / 16; ++i) {
            final CellField cf = new CellField();
            cf.left = stream.readInt();
            cf.top = stream.readInt();
            cf.right = stream.readInt();
            cf.bottom = stream.readInt();
            this.addMergeRect(cf);
        }
    }
    
    public void saveToStream(final Stream stream) throws StreamException {
        for (int i = 0; i < this.list.size(); ++i) {
            final CellField cf = this.list.get(i);
            stream.writeInt(cf.left);
            stream.writeInt(cf.top);
            stream.writeInt(cf.right);
            stream.writeInt(cf.bottom);
        }
    }
    
    public boolean getAutoMerge() {
        return this.autoMerge;
    }
    
    public void setAutoMerge(final boolean value) {
        this.autoMerge = value;
    }
}
